@@ -74,26 +74,31 @@ class GTI:
         reference_time : `~astropy.time.Time`
             the reference time to use in GTI definition
         """
-        start = Quantity(start)
-        stop = Quantity(stop)
+        start = np.atleast_1d(Quantity(start))
+        stop = np.atleast_1d(Quantity(stop))
         reference_time = Time(reference_time)
         meta = time_ref_to_dict(reference_time)
         table = Table({"START": start.to("s"), "STOP": stop.to("s")}, meta=meta)
         return cls(table)
 
     @classmethod
-    def read(cls, filename, **kwargs):
+    def read(cls, filename, hdu="GTI"):
         """Read from FITS file.
 
         Parameters
         ----------
         filename : `pathlib.Path`, str
             Filename
+        hdu : str
+            hdu name. Default GTI.
         """
         filename = make_path(filename)
-        kwargs.setdefault("hdu", "GTI")
-        table = Table.read(str(filename), **kwargs)
-        return cls(table=table)
+        table = Table.read(filename, hdu=hdu)
+        return cls(table)
+
+    def write(self, filename, **kwargs):
+        """Write to file."""
+        self.table.write(make_path(filename), **kwargs)
 
     def __str__(self):
         return (
