@@ -2,38 +2,34 @@
 import abc
 import copy
 import inspect
-import re
-from collections import OrderedDict
 import logging
 import numpy as np
-from scipy.interpolate import interp1d
-from astropy.utils.misc import InheritDocstrings
-from astropy.io import fits
+import scipy.interpolate
 from astropy import units as u
 from astropy.coordinates import SkyCoord
-from astropy.table import QTable, Column
-from ..utils.interpolation import interpolation_scale
-from .utils import find_hdu, find_bands_hdu, INVALID_INDEX
+from astropy.io import fits
+from astropy.table import Column, QTable, Table
+from astropy.utils import lazyproperty
+from gammapy.utils.interpolation import interpolation_scale
+from .utils import INVALID_INDEX, edges_from_lo_hi, find_bands_hdu, find_hdu
 
-__all__ = ["MapCoord", "MapGeom", "MapAxis"]
+__all__ = ["MapCoord", "Geom", "MapAxis"]
 
 log = logging.getLogger(__name__)
 
 
-def make_axes(axes_in, conv):
+def make_axes(axes_in):
     """Make a sequence of `~MapAxis` objects."""
     if axes_in is None:
         return []
 
     axes_out = []
-    for i, ax in enumerate(axes_in):
+    for idx, ax in enumerate(axes_in):
         if isinstance(ax, np.ndarray):
             ax = MapAxis(ax)
 
-        if conv in ["fgst-ccube", "fgst-template"]:
-            ax.name = "energy"
-        elif ax.name == "":
-            ax.name = "axis%i" % i
+        if ax.name == "":
+            ax.name = "axis{}".format(idx)
 
         axes_out += [ax]
 
@@ -81,71 +77,84 @@ def make_axes_cols(axes, axis_names=None):
     return cols
 
 
-def find_and_read_bands(hdu, header=None):
+def energy_axis_from_fgst_ccube(hdu):
+    bands = Table.read(hdu)
+    edges_min = bands["E_MIN"].quantity
+    edges_max = bands["E_MAX"].quantity
+    edges = edges_from_lo_hi(edges_min, edges_max)
+    return [MapAxis.from_edges(edges=edges, name="energy", interp="log")]
+
+
+def energy_axis_from_fgst_template(hdu):
+    bands = Table.read(hdu)
+
+    allowed_names = ["Energy", "ENERGY", "energy"]
+    for colname in bands.colnames:
+        if colname in allowed_names:
+            tag = colname
+            break
+
+    nodes = bands[tag].data
+
+    return [MapAxis.from_nodes(nodes=nodes, name="energy", unit="MeV", interp="log")]
+
+
+def axes_from_bands_hdu(hdu):
     """Read and returns the map axes from a BANDS table.
 
     Parameters
     ----------
     hdu : `~astropy.io.fits.BinTableHDU`
         The BANDS table HDU.
-    header : `~astropy.io.fits.Header`
-        Header
 
     Returns
     -------
     axes : list of `~MapAxis`
         List of axis objects.
     """
+    axes = []
+
+    bands = Table.read(hdu)
+
+    for idx in range(5):
+        axcols = bands.meta.get("AXCOLS{}".format(idx + 1))
+
+        if axcols is None:
+            break
+
+        colnames = axcols.split(",")
+        node_type = "edges" if len(colnames) == 2 else "center"
+
+        # TODO: check why this extra case is needed
+        if colnames[0] == "E_MIN":
+            name = "energy"
+        else:
+            name = colnames[0].split("_")[0].lower()
+        interp = bands.meta.get("INTERP{}".format(idx + 1), "lin")
+
+        if node_type == "center":
+            nodes = np.unique(bands[colnames[0]].quantity)
+        else:
+            edges_min = np.unique(bands[colnames[0]].quantity)
+            edges_max = np.unique(bands[colnames[1]].quantity)
+            nodes = edges_from_lo_hi(edges_min, edges_max)
+
+        axis = MapAxis(nodes=nodes, node_type=node_type, interp=interp, name=name)
+        axes.append(axis)
+
+    return axes
+
+
+def find_and_read_bands(hdu):
     if hdu is None:
         return []
 
-    axes = []
-    axis_cols = []
     if hdu.name == "ENERGIES":
-        axis_cols = [["ENERGY"]]
+        axes = energy_axis_from_fgst_template(hdu)
     elif hdu.name == "EBOUNDS":
-        axis_cols = [["E_MIN", "E_MAX"]]
+        axes = energy_axis_from_fgst_ccube(hdu)
     else:
-        for i in range(5):
-            if "AXCOLS%i" % i in hdu.header:
-                axis_cols += [hdu.header["AXCOLS%i" % i].split(",")]
-
-    interp = "lin"
-    for i, cols in enumerate(axis_cols):
-
-        if "ENERGY" in cols or "E_MIN" in cols:
-            name = "energy"
-            interp = "log"
-        elif re.search("(.+)_MIN", cols[0]):
-            name = re.search("(.+)_MIN", cols[0]).group(1)
-        else:
-            name = cols[0]
-
-        interp_header_key = "INTERP%i" % (i + 1)
-        if header is not None and interp_header_key in header:
-            interp_header_val = header[interp_header_key]
-            if interp_header_val in ["sqrt", "log", "lin"]:
-                interp = interp_header_val
-            else:
-                log.warning(
-                    "Invalid map axis interp: {!r}. Using default: {!r}".format(
-                        interp_header_val, interp
-                    )
-                )
-
-        unit = hdu.data.columns[cols[0]].unit
-        if unit is None and header is not None:
-            unit = header.get("CUNIT%i" % (3 + i), "")
-        if unit is None:
-            unit = ""
-        if len(cols) == 2:
-            xmin = np.unique(hdu.data.field(cols[0]))
-            xmax = np.unique(hdu.data.field(cols[1]))
-            nodes = np.append(xmin, xmax[-1])
-            axes.append(MapAxis(nodes, name=name, unit=unit, interp=interp))
-        else:
-            nodes = np.unique(hdu.data.field(cols[0]))
-            axes.append(MapAxis.from_nodes(nodes, name=name, unit=unit, interp=interp))
+        axes = axes_from_bands_hdu(hdu)
 
     return axes
 
@@ -161,16 +170,26 @@ def get_shape(param):
 
 
 def coordsys_to_frame(coordsys):
-    if coordsys in ["CEL", "C"]:
+    if coordsys in ["CEL", "C", "fk5", "fk4", "icrs"]:
         return "icrs"
-    elif coordsys in ["GAL", "G"]:
+    elif coordsys in ["GAL", "G", "galactic"]:
         return "galactic"
     else:
-        raise ValueError("Unrecognized coordinate system: {!r}".format(coordsys))
+        raise ValueError(f"Unrecognized coordinate system: {coordsys!r}")
 
 
+def frame_to_coordsys(frame):
+    if frame in ["fk5", "fk4", "icrs"]:
+        return "CEL"
+    elif frame == "galactic":
+        return "GAL"
+    else:
+        raise ValueError(f"Unrecognized coordinate system: {frame!r}")
+
+
+# TODO: remove (or improve)
 def skycoord_to_lonlat(skycoord, coordsys=None):
-    """
+    """Convert SkyCoord to lon, lat, frame.
 
     Returns
     -------
@@ -178,21 +197,12 @@ def skycoord_to_lonlat(skycoord, coordsys=None):
         Longitude in degrees.
     lat : `~numpy.ndarray`
         Latitude in degrees.
-    frame : str
-        Name of coordinate frame.
     """
-    if coordsys in ["CEL", "C"]:
-        skycoord = skycoord.transform_to("icrs")
-    elif coordsys in ["GAL", "G"]:
-        skycoord = skycoord.transform_to("galactic")
+    if coordsys:
+        frame = coordsys_to_frame(coordsys)
+        skycoord = skycoord.transform_to(frame)
 
-    frame = skycoord.frame.name
-    if frame in ["icrs", "fk5"]:
-        return skycoord.ra.deg, skycoord.dec.deg, frame
-    elif frame in ["galactic"]:
-        return skycoord.l.deg, skycoord.b.deg, frame
-    else:
-        raise ValueError("Unrecognized SkyCoord frame: {!r}".format(frame))
+    return skycoord.data.lon.deg, skycoord.data.lat.deg, skycoord.frame
 
 
 def lonlat_to_skycoord(lon, lat, coordsys):
@@ -200,20 +210,19 @@ def lonlat_to_skycoord(lon, lat, coordsys):
 
 
 def pix_tuple_to_idx(pix):
-    """Convert a tuple of pixel coordinate arrays to a tuple of pixel
-    indices.
+    """Convert a tuple of pixel coordinate arrays to a tuple of pixel indices.
 
     Pixel coordinates are rounded to the closest integer value.
 
     Parameters
     ----------
     pix : tuple
-        Tuple of pixel coordinates with one element for each dimension.
+        Tuple of pixel coordinates with one element for each dimension
 
     Returns
     -------
     idx : `~numpy.ndarray`
-        Array of pixel indices.
+        Array of pixel indices
     """
     idx = []
     for p in pix:
@@ -228,25 +237,6 @@ def pix_tuple_to_idx(pix):
     return tuple(idx)
 
 
-def axes_pix_to_coord(axes, pix):
-    """Perform pixel to axis coordinates for a list of `~MapAxis`
-    objects.
-
-    Parameters
-    ----------
-    axes : list
-        List of `~MapAxis`.
-
-    pix : tuple
-        Tuple of pixel coordinates.
-    """
-    coords = []
-    for ax, t in zip(axes, pix):
-        coords += [ax.pix_to_coord(t)]
-
-    return coords
-
-
 def coord_to_idx(edges, x, clip=False):
     """Convert axis coordinates ``x`` to bin indices.
 
@@ -266,16 +256,11 @@ def coord_to_idx(edges, x, clip=False):
     return ibin
 
 
-def bin_to_val(edges, bins):
-    ctr = 0.5 * (edges[1:] + edges[:-1])
-    return ctr[bins]
-
-
 def coord_to_pix(edges, coord, interp="lin"):
     """Convert axis to pixel coordinates for given interpolation scheme."""
     scale = interpolation_scale(interp)
 
-    interp_fn = interp1d(
+    interp_fn = scipy.interpolate.interp1d(
         scale(edges), np.arange(len(edges), dtype=float), fill_value="extrapolate"
     )
 
@@ -286,7 +271,7 @@ def pix_to_coord(edges, pix, interp="lin"):
     """Convert pixel to grid coordinates for given interpolation scheme."""
     scale = interpolation_scale(interp)
 
-    interp_fn = interp1d(
+    interp_fn = scipy.interpolate.interp1d(
         np.arange(len(edges), dtype=float), scale(edges), fill_value="extrapolate"
     )
 
@@ -326,19 +311,8 @@ class MapAxis:
         String specifying the data units.
     """
 
-    __slots__ = [
-        "_name",
-        "_nodes",
-        "_node_type",
-        "_interp",
-        "_pix_offset",
-        "_nbin",
-        "_unit",
-    ]
-
     # TODO: Add methods to faciliate FITS I/O.
     # TODO: Cache an interpolation object?
-
     def __init__(self, nodes, interp="lin", name="", node_type="edges", unit=""):
 
         self.name = name
@@ -348,22 +322,23 @@ class MapAxis:
         if ~(np.all(nodes == np.sort(nodes)) or np.all(nodes[::-1] == np.sort(nodes))):
             raise ValueError("MapAxis: node values must be sorted")
 
+        if len(nodes) == 1 and node_type == "center":
+            raise ValueError("Single bins can only be used with node-type 'edges'")
+
         if isinstance(nodes, u.Quantity):
             unit = nodes.unit if nodes.unit is not None else ""
             nodes = nodes.value
         else:
             nodes = np.array(nodes)
 
-        self.unit = unit
+        self._unit = u.Unit(unit)
         self._nodes = nodes
         self._node_type = node_type
         self._interp = interp
 
         if (self._nodes < 0).any() and interp != "lin":
             raise ValueError(
-                "Interpolation scaling '{}' only support for positive node values.".format(
-                    interp
-                )
+                f"Interpolation scaling {interp!r} only support for positive node values."
             )
 
         # Set pixel coordinate of first node
@@ -374,12 +349,11 @@ class MapAxis:
             self._pix_offset = 0.0
             nbin = len(nodes)
         else:
-            raise ValueError("Invalid node type: {!r}".format(node_type))
+            raise ValueError(f"Invalid node type: {node_type!r}")
 
         self._nbin = nbin
 
     def __eq__(self, other):
-        """Test axis equality. Absolute and relative tolerances of 1e-6 are used"""
         if not isinstance(other, self.__class__):
             return NotImplemented
 
@@ -400,6 +374,9 @@ class MapAxis:
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    def __hash__(self):
+        return id(self)
+
     @property
     def interp(self):
         """Interpolation scale of the axis."""
@@ -414,18 +391,23 @@ class MapAxis:
     def name(self, val):
         self._name = val
 
-    @property
+    @lazyproperty
     def edges(self):
         """Return array of bin edges."""
         pix = np.arange(self.nbin + 1, dtype=float) - 0.5
         return u.Quantity(self.pix_to_coord(pix), self._unit, copy=False)
 
-    @property
+    @lazyproperty
     def center(self):
         """Return array of bin centers."""
         pix = np.arange(self.nbin, dtype=float)
         return u.Quantity(self.pix_to_coord(pix), self._unit, copy=False)
 
+    @lazyproperty
+    def bin_width(self):
+        """Array of bin widths."""
+        return np.diff(self.edges)
+
     @property
     def nbin(self):
         """Return number of bins."""
@@ -441,10 +423,6 @@ class MapAxis:
         """Return coordinate axis unit."""
         return self._unit
 
-    @unit.setter
-    def unit(self, val):
-        self._unit = u.Unit(val)
-
     @classmethod
     def from_bounds(cls, lo_bnd, hi_bnd, nbin, **kwargs):
         """Generate an axis object from a lower/upper bound and number of bins.
@@ -474,7 +452,7 @@ class MapAxis:
         elif node_type == "center":
             nnode = nbin
         else:
-            raise ValueError("Invalid node type: {!r}".format(node_type))
+            raise ValueError(f"Invalid node type: {node_type!r}")
 
         if interp == "lin":
             nodes = np.linspace(lo_bnd, hi_bnd, nnode)
@@ -483,10 +461,45 @@ class MapAxis:
         elif interp == "sqrt":
             nodes = np.linspace(lo_bnd ** 0.5, hi_bnd ** 0.5, nnode) ** 2.0
         else:
-            raise ValueError("Invalid interp: {}".format(interp))
+            raise ValueError(f"Invalid interp: {interp}")
 
         return cls(nodes, **kwargs)
 
+    @classmethod
+    def from_energy_bounds(cls, emin, emax, nbin, unit=None):
+        """Make an energy axis.
+
+        Used frequently also to make energy grids, by making
+        the axis, and then using ``axis.center`` or ``axis.edges``.
+
+        Parameters
+        ----------
+        emin, emax : `~astropy.units.Quantity`, float
+            Energy range
+        nbin : int
+            Number of bins
+        unit : `~astropy.units.Unit`
+            Energy unit
+
+        Returns
+        -------
+        axis : `MapAxis`
+            Axis with name "energy" and interp "log".
+        """
+        if unit is None:
+            emin = u.Quantity(emin)
+            emax = u.Quantity(emax)
+            unit = emax.unit
+            emin = emin.to(unit)
+        else:
+            unit = u.Unit(unit)
+            emin = u.Quantity(emin, unit)
+            emax = u.Quantity(emax, unit)
+
+        return cls.from_bounds(
+            emin.value, emax.value, nbin=nbin, unit=unit, interp="log", name="energy"
+        )
+
     @classmethod
     def from_nodes(cls, nodes, **kwargs):
         """Generate an axis object from a sequence of nodes (bin centers).
@@ -544,7 +557,8 @@ class MapAxis:
             Array of axis coordinate values.
         """
         pix = pix - self._pix_offset
-        return pix_to_coord(self._nodes, pix, interp=self._interp)
+        values = pix_to_coord(self._nodes, pix, interp=self._interp)
+        return u.Quantity(values, unit=self.unit, copy=False)
 
     def coord_to_pix(self, coord):
         """Transform from axis to pixel coordinates.
@@ -639,8 +653,8 @@ class MapAxis:
         str_ += fmt.format("nbins", str(self.nbin))
         str_ += fmt.format("node type", self.node_type)
         vals = self.edges if self.node_type == "edges" else self.center
-        str_ += fmt.format("{} min".format(self.node_type), "{:.1e}".format(vals.min()))
-        str_ += fmt.format("{} max".format(self.node_type), "{:.1e}".format(vals.max()))
+        str_ += fmt.format(f"{self.node_type} min", "{:.1e}".format(vals.min()))
+        str_ += fmt.format(f"{self.node_type} max", "{:.1e}".format(vals.max()))
         str_ += fmt.format("interp", self._interp)
         return str_
 
@@ -692,11 +706,11 @@ class MapAxis:
         edges_pix = np.clip(edges_pix, -0.5, self.nbin - 0.5)
         edges_idx = np.round(edges_pix + 0.5) - 0.5
         edges_idx = np.unique(edges_idx)
-        edges_ref = self.pix_to_coord(edges_idx) * self.unit
+        edges_ref = self.pix_to_coord(edges_idx)
 
         groups = QTable()
-        groups["{}_min".format(self.name)] = edges_ref[:-1]
-        groups["{}_max".format(self.name)] = edges_ref[1:]
+        groups[f"{self.name}_min"] = edges_ref[:-1]
+        groups[f"{self.name}_max"] = edges_ref[1:]
 
         groups["idx_min"] = (edges_idx[:-1] + 0.5).astype(int)
         groups["idx_max"] = (edges_idx[1:] - 0.5).astype(int)
@@ -712,8 +726,8 @@ class MapAxis:
                 "bin_type": "underflow",
                 "idx_min": 0,
                 "idx_max": edge_idx_start,
-                "{}_min".format(self.name): self.pix_to_coord(-0.5) * self.unit,
-                "{}_max".format(self.name): edge_ref_start,
+                f"{self.name}_min": self.pix_to_coord(-0.5),
+                f"{self.name}_max": edge_ref_start,
             }
             groups.insert_row(0, vals=underflow)
 
@@ -724,9 +738,8 @@ class MapAxis:
                 "bin_type": "overflow",
                 "idx_min": edge_idx_end + 1,
                 "idx_max": self.nbin - 1,
-                "{}_min".format(self.name): edge_ref_end,
-                "{}_max".format(self.name): self.pix_to_coord(self.nbin - 0.5)
-                * self.unit,
+                f"{self.name}_min": edge_ref_end,
+                f"{self.name}_max": self.pix_to_coord(self.nbin - 0.5),
             }
             groups.add_row(vals=overflow)
 
@@ -799,7 +812,7 @@ class MapCoord:
 
     Parameters
     ----------
-    data : `~collections.OrderedDict` of `~numpy.ndarray`
+    data : `dict` of `~numpy.ndarray`
         Dictionary of coordinate arrays.
     coordsys : {'CEL', 'GAL', None}
         Spatial coordinate system.  If None then the coordinate system
@@ -814,16 +827,9 @@ class MapCoord:
         if "lon" not in data or "lat" not in data:
             raise ValueError("data dictionary must contain axes named 'lon' and 'lat'.")
 
-        if issubclass(data["lon"].__class__, u.Quantity):
-            raise ValueError("Quantity not supported for 'lon'")
-        if issubclass(data["lat"].__class__, u.Quantity):
-            raise ValueError("Quantity not supported for 'lat'")
-
-        data = OrderedDict(
-            [(k, np.atleast_1d(np.asanyarray(v))) for k, v in data.items()]
-        )
+        data = {k: np.atleast_1d(np.asanyarray(v)) for k, v in data.items()}
         vals = np.broadcast_arrays(*data.values(), subok=True)
-        self._data = OrderedDict(zip(data.keys(), vals))
+        self._data = dict(zip(data.keys(), vals))
         self._coordsys = coordsys
         self._match_by_name = match_by_name
 
@@ -862,24 +868,24 @@ class MapCoord:
 
     @property
     def theta(self):
-        """Theta co-latitude angle in radians"""
-        return np.pi / 2.0 - np.radians(self.lat)
+        """Theta co-latitude angle in radians."""
+        theta = u.Quantity(self.lat, unit="deg", copy=False).to_value("rad")
+        return np.pi / 2.0 - theta
 
     @property
     def phi(self):
-        """Phi longitude angle in radians"""
-        return np.radians(self.lon)
+        """Phi longitude angle in radians."""
+        phi = u.Quantity(self.lon, unit="deg", copy=False).to_value("rad")
+        return phi
 
     @property
     def coordsys(self):
-        """Coordinate system (str)"""
+        """Coordinate system (str)."""
         return self._coordsys
 
     @property
     def match_by_name(self):
-        """Boolean flag indicating whether axis lookup should be performed by
-        name (True) or index (False).
-        """
+        """Boolean flag: axis lookup by name (True) or index (False)."""
         return self._match_by_name
 
     @property
@@ -905,9 +911,9 @@ class MapCoord:
             A coordinates object.
         """
         if isinstance(coords, (list, tuple)):
-            coords_dict = OrderedDict([("lon", coords[0]), ("lat", coords[1])])
+            coords_dict = {"lon": coords[0], "lat": coords[1]}
             for i, c in enumerate(coords[2:]):
-                coords_dict["axis{}".format(i)] = c
+                coords_dict[f"axis{i}"] = c
         else:
             raise ValueError("Unrecognized input type.")
 
@@ -928,15 +934,9 @@ class MapCoord:
             the `~astropy.coordinates.SkyCoord` object.
         """
         skycoord = coords[0]
-        name = skycoord.frame.name
-        if name in ["icrs", "fk5"]:
-            coords = (skycoord.ra.deg, skycoord.dec.deg) + coords[1:]
-            coords = cls._from_lonlat(coords, coordsys="CEL")
-        elif name in ["galactic"]:
-            coords = (skycoord.l.deg, skycoord.b.deg) + coords[1:]
-            coords = cls._from_lonlat(coords, coordsys="GAL")
-        else:
-            raise ValueError("Unrecognized coordinate frame: {!r}".format(name))
+        coordsys_skycoord = frame_to_coordsys(skycoord.frame.name)
+        coords = (skycoord.data.lon.deg, skycoord.data.lat.deg) + coords[1:]
+        coords = cls._from_lonlat(coords, coordsys=coordsys_skycoord)
 
         if coordsys is None:
             return coords
@@ -951,7 +951,7 @@ class MapCoord:
         elif isinstance(coords[0], SkyCoord):
             return cls._from_skycoord(coords, coordsys=coordsys)
         else:
-            raise TypeError("Type not supported: {!r}".format(type(coords)))
+            raise TypeError(f"Type not supported: {type(coords)!r}")
 
     @classmethod
     def _from_dict(cls, coords, coordsys=None):
@@ -959,14 +959,14 @@ class MapCoord:
         if "lon" in coords and "lat" in coords:
             return cls(coords, coordsys=coordsys)
         elif "skycoord" in coords:
-            coords_dict = OrderedDict()
             lon, lat, frame = skycoord_to_lonlat(coords["skycoord"], coordsys=coordsys)
-            coords_dict["lon"] = lon
-            coords_dict["lat"] = lat
+            coords_dict = {"lon": lon, "lat": lat}
             for k, v in coords.items():
                 if k == "skycoord":
                     continue
                 coords_dict[k] = v
+            if coordsys is None:
+                coordsys = frame_to_coordsys(frame.name)
             return cls(coords_dict, coordsys=coordsys)
         else:
             raise ValueError("coords dict must contain 'lon'/'lat' or 'skycoord'.")
@@ -1014,7 +1014,7 @@ class MapCoord:
         elif isinstance(data, SkyCoord):
             return cls._from_skycoord((data,), coordsys=coordsys)
         else:
-            raise TypeError("Unsupported input type: {!r}".format(type(data)))
+            raise TypeError(f"Unsupported input type: {type(data)!r}")
 
     def to_coordsys(self, coordsys):
         """Convert to a different coordinate frame.
@@ -1035,6 +1035,12 @@ class MapCoord:
             skycoord = lonlat_to_skycoord(self.lon, self.lat, self.coordsys)
             lon, lat, frame = skycoord_to_lonlat(skycoord, coordsys=coordsys)
             data = copy.deepcopy(self._data)
+            if isinstance(self.lon, u.Quantity):
+                lon = u.Quantity(lon, unit="deg", copy=False)
+
+            if isinstance(self.lon, u.Quantity):
+                lat = u.Quantity(lat, unit="deg", copy=False)
+
             data["lon"] = lon
             data["lat"] = lat
             return self.__class__(data, coordsys, self._match_by_name)
@@ -1052,7 +1058,7 @@ class MapCoord:
         coords : `~MapCoord`
             A coordinates object.
         """
-        data = OrderedDict([(k, v[mask]) for k, v in self._data.items()])
+        data = {k: v[mask] for k, v in self._data.items()}
         return self.__class__(data, self.coordsys, self._match_by_name)
 
     def copy(self):
@@ -1060,48 +1066,20 @@ class MapCoord:
         return copy.deepcopy(self)
 
     def __repr__(self):
-        str_ = self.__class__.__name__
-        str_ += "\n\n"
-        str_ += "\taxes     : {}\n".format(", ".join(self._data.keys()))
-        str_ += "\tshape    : {}\n".format(self.shape[::-1])
-        str_ += "\tndim     : {}\n".format(self.ndim)
-        str_ += "\tcoordsys : {}\n".format(self.coordsys)
-        return str_
-
-    # TODO: this is a temporary solution until we have decided how to handle
-    # quantities uniformly. This should be called after any `MapCoord.create()`
-    # to support that users can pass quantities in any Map.xxx_by_coord() method.
-    def match_axes_units(self, geom):
-        """Match the units of the non-spatial axes to a given map geometry.
-
-        Parameters
-        ----------
-        geom : `MapGeom`
-            Map geometry with specified units per axis.
-
-        Returns
-        -------
-        coords : `MapCoord`
-            Map coord object with matched units
-        """
-        coords = OrderedDict()
-
-        for name, coord in self._data.items():
-            if name in ["lon", "lat"]:
-                coords[name] = coord
-            else:
-                ax = geom.get_axis_by_name(name)
-                coords[name] = u.Quantity(coord, ax.unit, copy=False).value
-
-        return self.__class__(coords, coordsys=self.coordsys)
+        return (
+            f"{self.__class__.__name__}\n\n"
+            f"\taxes     : {list(self._data.keys())}\n"
+            f"\tshape    : {self.shape[::-1]}\n"
+            f"\tndim     : {self.ndim}\n"
+            f"\tcoordsys : {self.coordsys}\n"
+        )
 
 
-class MapGeomMeta(InheritDocstrings, abc.ABCMeta):
-    pass
+class Geom(abc.ABC):
+    """Map geometry base class.
 
-
-class MapGeom(metaclass=MapGeomMeta):
-    """Base class for WCS and HEALPix geometries."""
+    See also: `~gammapy.maps.WcsGeom` and `~gammapy.maps.HpxGeom`
+    """
 
     @property
     @abc.abstractmethod
@@ -1146,7 +1124,7 @@ class MapGeom(metaclass=MapGeomMeta):
 
         Returns
         -------
-        geom : `~MapGeom`
+        geom : `~Geom`
             Geometry object.
         """
         if hdu is None:
@@ -1163,7 +1141,6 @@ class MapGeom(metaclass=MapGeomMeta):
         return cls.from_header(hdu.header, hdu_bands)
 
     def make_bands_hdu(self, hdu=None, hdu_skymap=None, conv=None):
-        conv = self.conv if conv is None else conv
         header = fits.Header()
         self._fill_header_from_axes(header)
         axis_names = None
@@ -1179,7 +1156,7 @@ class MapGeom(metaclass=MapGeomMeta):
             axis_names = ["energy"]
         elif conv == "gadf" and hdu is None:
             if hdu_skymap:
-                hdu = "{}_{}".format(hdu_skymap, "BANDS")
+                hdu = f"{hdu_skymap}_BANDS"
             else:
                 hdu = "BANDS"
         # else:
@@ -1316,8 +1293,9 @@ class MapGeom(metaclass=MapGeomMeta):
 
     @abc.abstractmethod
     def pix_to_idx(self, pix, clip=False):
-        """Convert pixel coordinates to pixel indices.  Returns -1 for pixel
-        coordinates that lie outside of the map.
+        """Convert pixel coordinates to pixel indices.
+
+        Returns -1 for pixel coordinates that lie outside of the map.
 
         Parameters
         ----------
@@ -1368,8 +1346,7 @@ class MapGeom(metaclass=MapGeomMeta):
         return np.all(np.stack([t != INVALID_INDEX.int for t in idx]), axis=0)
 
     def slice_by_idx(self, slices):
-        """Create a new geometry by cutting in the non-spatial dimensions of
-        this geometry.
+        """Create a new geometry by slicing the non-spatial axes.
 
         Parameters
         ----------
@@ -1381,7 +1358,7 @@ class MapGeom(metaclass=MapGeomMeta):
 
         Returns
         -------
-        geom : `~MapGeom`
+        geom : `~Geom`
             Sliced geometry.
         """
         axes = []
@@ -1396,20 +1373,20 @@ class MapGeom(metaclass=MapGeomMeta):
 
     @abc.abstractmethod
     def to_image(self):
-        """Create a 2D geometry by dropping all non-spatial dimensions of this
-        geometry.
+        """Create 2D image geometry (drop non-spatial dimensions).
 
         Returns
         -------
-        geom : `~MapGeom`
+        geom : `~Geom`
             Image geometry.
         """
         pass
 
     @abc.abstractmethod
     def to_cube(self, axes):
-        """Create a new geometry by appending a list of non-spatial axes to
-        the present geometry.  This will result in a new geometry with
+        """Append non-spatial axes to create a higher-dimensional geometry.
+
+        This will result in a new geometry with
         N+M dimensions where N is the number of current dimensions and
         M is the number of axes in the list.
 
@@ -1420,7 +1397,7 @@ class MapGeom(metaclass=MapGeomMeta):
 
         Returns
         -------
-        geom : `~MapGeom`
+        geom : `~Geom`
             Map geometry.
         """
         pass
@@ -1456,7 +1433,7 @@ class MapGeom(metaclass=MapGeomMeta):
 
         Returns
         -------
-        geom : `~MapGeom`
+        geom : `~Geom`
             Padded geometry.
         """
         pass
@@ -1473,7 +1450,7 @@ class MapGeom(metaclass=MapGeomMeta):
 
         Returns
         -------
-        geom : `~MapGeom`
+        geom : `~Geom`
             Cropped geometry.
         """
         pass
@@ -1491,7 +1468,7 @@ class MapGeom(metaclass=MapGeomMeta):
 
         Returns
         -------
-        geom : `~MapGeom`
+        geom : `~Geom`
             Downsampled geometry.
 
         """
@@ -1510,7 +1487,7 @@ class MapGeom(metaclass=MapGeomMeta):
 
         Returns
         -------
-        geom : `~MapGeom`
+        geom : `~Geom`
             Upsampled geometry.
 
         """
@@ -1530,11 +1507,11 @@ class MapGeom(metaclass=MapGeomMeta):
             elif ax.name == "energy" and ax.node_type == "center":
                 header[key] = "ENERGY"
             elif ax.node_type == "edges":
-                header[key] = "{}_MIN,{}_MAX".format(name, name)
+                header[key] = f"{name}_MIN,{name}_MAX"
             elif ax.node_type == "center":
                 header[key] = name
             else:
-                raise ValueError("Invalid node type {!r}".format(ax.node_type))
+                raise ValueError(f"Invalid node type {ax.node_type!r}")
 
             key_interp = "INTERP%i" % idx
             header[key_interp] = ax._interp
@@ -1591,7 +1568,7 @@ class MapGeom(metaclass=MapGeomMeta):
         return self.__class__(**kwargs)
 
     def copy(self, **kwargs):
-        """Copy `MapGeom` instance and overwrite given attributes.
+        """Copy and overwrite given attributes.
 
         Parameters
         ----------
@@ -1600,7 +1577,7 @@ class MapGeom(metaclass=MapGeomMeta):
 
         Returns
         -------
-        copy : `MapGeom`
+        copy : `Geom`
             Copied map geometry.
         """
         return self._init_copy(**kwargs)
@@ -1623,6 +1600,6 @@ class MapGeom(metaclass=MapGeomMeta):
 
         # create mask
         coords = self.get_coord()
-        mask = coords["energy"] > emin.to_value(energy_axis.unit)
-        mask &= coords["energy"] < emax.to_value(energy_axis.unit)
+        mask = coords["energy"] > emin
+        mask &= coords["energy"] < emax
         return mask
